@@ -103,15 +103,17 @@ class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
 }
 
+#define CAMEL_EXCHANGE_SERVER_FLAGS \
+	(CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_ANSWERED_ALL | \
+	 CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_DRAFT | CAMEL_MESSAGE_SEEN)
+
 static void
 init (CamelFolder *folder)
 {
 	folder->folder_flags = CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY | CAMEL_FOLDER_HAS_SEARCH_CAPABILITY;
 	folder->permanent_flags =
-		CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_ANSWERED_ALL |
-		CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_DRAFT |
-		CAMEL_MESSAGE_FLAGGED | CAMEL_MESSAGE_SEEN |
-		CAMEL_MESSAGE_USER;
+		CAMEL_EXCHANGE_SERVER_FLAGS | CAMEL_MESSAGE_FLAGGED |
+		CAMEL_MESSAGE_JUNK | CAMEL_MESSAGE_USER;
 }
 
 static void
@@ -624,7 +626,7 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 {
 	CamelExchangeFolder *exch_source = CAMEL_EXCHANGE_FOLDER (source);
 	CamelExchangeFolder *exch_dest = CAMEL_EXCHANGE_FOLDER (dest);
-	GPtrArray *ret_uids;
+	GPtrArray *ret_uids = NULL;
 	int hier_len;
 
 	camel_operation_start (NULL, delete_originals ? _("Moving messages") :
@@ -647,7 +649,9 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			     CAMEL_STUB_ARG_RETURN,
 			     CAMEL_STUB_ARG_STRINGARRAY, &ret_uids,
 			     CAMEL_STUB_ARG_END)) {
-		cache_xfer (exch_source, exch_dest, uids, ret_uids, FALSE);
+		if (ret_uids->len != 0)
+			cache_xfer (exch_source, exch_dest, uids, ret_uids, FALSE);
+
 		if (transferred_uids)
 			*transferred_uids = ret_uids;
 		else {
@@ -810,7 +814,7 @@ camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
 	if (!info)
 		return;
 
-	flags |= (info->flags & (CAMEL_MESSAGE_ATTACHMENTS | CAMEL_MESSAGE_FLAGGED));
+	flags |= (info->flags & ~CAMEL_EXCHANGE_SERVER_FLAGS);
 
 	if (info->flags != flags) {
 		info->flags = flags;
@@ -925,7 +929,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		for (i = 0; i < summary->len; i++) {
 			info = summary->pdata[i];
 			uids->pdata[i] = (char *)camel_message_info_uid (info);
-			flags->data[i] = info->flags & ~CAMEL_MESSAGE_ATTACHMENTS;
+			flags->data[i] = info->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
 			camel_tag_list_free (&info->user_tags);
 		}
 
